#This is the basic file to run the criteria for lines and triangles.

#################################
# 1. Preliminaries
#################################

#We introduce two indeterminates "q" and "r"; which will be our generic parameters

q:=Indeterminate(Rationals,"q");
r:=Indeterminate(Rationals,"r");

#We denote by "K" the field of rational functions on those two variables. 

K:=Field(q,r);;

#"F1" (also, later F(1)) and "F0" will denote the unit and the null element of the field "K"
F1:=One(K);;
F0:=Zero(K);;

#we define our roots in the field K, they shall be called "F()" instead of E():

F:=n->E(n)*F1;

#We denote by "R(n)" the set of roots in K of order n

R:=function(n)
local l,k;
l:=[];
for k in [1..n] do
if Order(F(n)^k)=n then
Add(l,F(n)^k);
fi;
od;
return l;
end;

#We collect in "C(n)" the set of roots of order leeser or equal than n
C:=function(n)
local l,k;
l:=[];
for k in [1..n] do
if F(n)^k<>F1 then
Add(l,F(n)^k);
fi;
od;
return l;
end;

#The set of roots we need to check.

Gf:=Set(Concatenation(C(14),C(18),C(20),C(24),C(30)));;

#We let "Rp" be the set of roots in F of order at most 100.


Rp:=[];;
for n in [1..100] do
Append(Rp,R(n));
od;



#################################
# 2. The list
#################################


##############################
#the function "check" returns the name of a given diagram in the list of "finite" diagrams of rank two in [H].
#the nth diagram of the kth row is named dk.n. 
#It returns 0 if the input diagram is not in the list.
#It admits mirrored diagrams.

check:=function(d)
if d[2]=F(1) then
return
"d1";
elif d[1]=d[3] and d[1]*d[2]=F(1) and d[2]<>F(1) then
return
"d2";
elif 
(d[1]*d[2]=F(1) and d[3]=-F(1) and d[2]<>F(1) and d[2]<>F(2)) 
or 
(d[2]*d[3]=F(1) and d[1]=-F(1) and d[2]<>F(1) and d[2]<>F(2))
then
return
"d3.1";
elif 
(d[1]=d[3] and d[3]=-F(1) and d[2]<>F(1) and d[2]<>F(2))
then
return
"d3.2";
elif
(d[1]^2*d[2]=F(1) and d[2]*d[3]=F(1) and d[1]<>F(1) and d[1]<>F(2)) 
or
(d[1]*d[2]=F(1) and d[2]*d[3]^2=F(1) and d[3]<>F(1) and d[3]<>F(2))
then
return
"d4";
elif 
(d[1]^2*d[2]=F(1) and d[3]=-F(1) and d[1]<>F(1) and d[1]<>F(2) and d[1]<>F(4) and d[1]<>-F(4)) 
or
(d[3]^2*d[2]=F(1) and d[1]=-F(1) and d[3]<>F(1) and d[3]<>F(2) and d[3]<>F(4) and d[3]<>-F(4))
then
return
"d5";
elif 
(d[1] in R(3) and d[2]*d[3]=F(1) and d[3]<>F(1) and d[3]<>F(3) and d[3]<>F(3)^2) 
or
(d[3] in R(3) and d[2]*d[1]=F(1) and d[1]<>F(1) and d[1]<>F(3) and d[1]<>F(3)^2)
then
return
"d6";
elif 
(d[1] in R(3) and d[1]=-d[2] and d[3]=-F(1)) 
or
(d[3] in R(3) and d[3]=-d[2] and d[1]=-F(1)) 
then
return
"d7";
elif
(d[1]*d[3]=F(1) and d[1] in R(3) and d[1]*d[2] in R(12))
then 
return
"d8.1";
Print(d[1]*d[2]);
elif
(d[2] in R(12) and d[1]=-d[2]^2 and d[3]=-F(1))
or
(d[2] in R(12) and d[3]=-d[2]^2 and d[1]=-F(1))
then 
return
"d8.2";
Print(d[2]^-1);
elif
(-d[2] in R(12) and d[1]=-d[2]^2 and d[3]=-F(1))
or
(-d[2] in R(12) and d[3]=-d[2]^2 and d[1]=-F(1))
then 
return
"d8.3";
Print(-d[2]);
elif
(d[2] in R(12) and d[1]=-d[2]^3 and d[3]=-F(1))
or
(d[2] in R(12) and d[3]=-d[2]^3 and d[1]=-F(1))
then 
return
"d8.4";
Print(d[2]);
elif
(-d[2] in R(12) and d[1]=d[2]^-3 and d[3]=-F(1))
or
(d[2] in R(12) and d[3]=d[2]^-3 and d[1]=-F(1))
then 
return
"d8.5";
Print(-d[2]^-1);
elif
(d[2] in R(12) and d[1]=-d[2]^2 and d[3]=d[1])
then 
return
"d9.1";
Print(d[2]);
elif
(-d[2]*d[1]^-1 in R(12) and d[2]=-d[2]^3*d[1]^-3 and d[3]=-F(1))
or
(-d[2]*d[3]^-1 in R(12) and d[2]=-d[2]^3*d[3]^-3 and d[1]=-F(1))
then 
return
"d9.2";
Print(-d[2]*d[1]^-1);
elif
(d[1] in R(12) and d[2]=d[1]^-3 and d[3]=-F(1))
or
(d[3] in R(12) and d[2]=d[3]^-3 and d[1]=-F(1))
then 
return
"d9.3";
Print(-d[1]^-1);
elif 
(d[1] in R(18) and d[2]=d[1]^-2 and d[3]=-d[1]^3)
or 
(d[3] in R(18) and d[2]=d[3]^-2 and d[1]=-d[3]^3)
then 
return
"d10.1";
elif
(d[2] in R(9) and d[1]=d[2]^-3 and d[3]=-F(1))
or
(d[2] in R(9) and d[3]=d[2]^-3 and d[1]=-F(1))
then 
return
"d10.2";
elif
(d[2] in R(9) and d[1]=-d[2]^2 and d[3]=-F(1))
or
(d[2] in R(9) and d[3]=-d[2]^2 and d[1]=-F(1))
then 
return
"d10.3";
elif 
(d[1]^3*d[2]=F(1) and d[3]*d[2]=F(1) and d[1]<>F(1) and d[1]<>F(2) and d[1]<>F(3) and d[1]<>F(3)^2)
or 
(d[3]^3*d[2]=F(1) and d[1]*d[2]=F(1) and d[3]<>F(1) and d[3]<>F(2) and d[3]<>F(3) and d[3]<>F(3)^2)
then
return
"d11";
elif
(d[2] in R(8) and d[1]=d[2]^2 and d[3]=d[2]^-1)
or 
(d[2] in R(8) and d[3]=d[2]^2 and d[1]=d[2]^-1)
then 
return
"d12.1";
Print(d[2]);
elif
(d[2] in R(8) and d[1]=d[2]^-2 and d[3]=-F(1))
or 
(d[2] in R(8) and d[3]=d[2]^-2 and d[1]=-F(1))
then 
return
"d12.2";
elif
(d[1] in R(8) and d[2]=-d[1] and d[3]=-F(1))
or 
(d[3] in R(8) and d[2]=-d[3] and d[1]=-F(1))
then 
return
"d12.3";
Print(-d[2]);
elif
(d[2] in R(24) and d[1]=d[2]^-6 and d[3]=-d[2]^4)
or 
(d[2] in R(24) and d[3]=d[2]^-6 and d[1]=-d[2]^4)
then 
return
"d13.1";
elif 
(d[2] in R(24) and d[1]=d[2]^6 and d[3]=d[2]^-1)
or 
(d[2] in R(24) and d[3]=d[2]^6 and d[1]=d[2]^-1)
then 
return
"d13.2";
Print(d[2]);
elif 
(d[2] in R(24) and d[1]=-d[2]^4 and d[3]=-F(1))
or 
(d[2] in R(24) and d[3]=-d[2]^4 and d[1]=-F(1))
then 
return
"d13.3";
Print(d[2]^5);
elif 
(d[1] in R(24) and d[2]=d[1]^-5 and d[3]=-F(1))
or 
(d[3] in R(24) and d[2]=d[3]^-5 and d[1]=-F(1))
then 
return
"d13.4";
elif
(d[1] in R(5) and d[2]=d[1]^2 and d[3]=-F(1))
or 
(d[3] in R(5) and d[2]=d[3]^2 and d[1]=-F(1))
then 
return
"d14.1";
elif
(-d[1] in R(5) and d[2]=-d[1] and d[3]=-F(1))
or 
(-d[3] in R(5) and d[2]=-d[3] and d[1]=-F(1))
then
return
"d14.2";
Print(d[2]^-3);
elif
(d[1] in R(20) and d[2]=d[1]^-3 and d[3]=-F(1))
or 
(d[3] in R(20) and d[2]=d[3]^-3 and d[1]=-F(1))
then 
return
"d15.1";
elif
(d[2] in R(20) and d[1]=-d[2]^6 and d[3]=-F(1))
or 
(d[2] in R(20) and d[3]=-d[2]^6 and d[1]=-F(1))
then 
return
"d15.3";
Print("-d[2]^7");
elif
(d[1] in R(30) and d[2]=d[1]^-3 and d[3]=d[1]^20)
or 
(d[3] in R(30) and d[2]=d[3]^-3 and d[1]=d[3]^20)
then 
return
"d16.1";
elif
(d[2] in R(30) and d[1]=d[2]^12 and d[3]=d[2]^-1)
or 
(d[2] in R(30) and d[3]=d[2]^12 and d[1]=d[2]^-1)
then 
return
"d16.2";
elif
(d[2] in R(30) and d[1]=-d[2]^5 and d[3]=-F(1))
or 
(d[2] in R(30) and d[3]=-d[2]^5 and d[1]=-F(1))
then
return
"d16.3";
elif
(d[2] in R(30) and d[1]=d[2]^-6 and d[3]=-F(1))
or 
(d[2] in R(30) and d[3]=d[2]^-6 and d[1]=-F(1))
then
return
"d16.4";
elif
(-d[1] in R(7) and d[2]=d[1]^-3 and d[3]=-F(1))
or 
(-d[3] in R(7) and d[2]=d[3]^-3 and d[1]=-F(1))
then 
return
"d17.1";
elif
(d[2]*d[1] in R(7) and d[2]=-d[2]^3*d[1]^3 and d[3]=-F(1))
or 
(d[2]*d[3] in R(7) and d[2]=-d[2]^3*d[3]^3 and d[1]=-F(1))
then 
return
"d17.2";
else 
return 0;
fi;
end;

#The function "Hcheck" returns 1 if the input diagram (of rank 2) is on the list; returns 0 otherwise. 

Hcheck:=function(d)
if check(d)<>0 then
return
1;
else 
return 
0;
fi;
end;




#####################################################################################

#the function "checkrb" returns the name of a given diagram in the list of "finite" diagrams of rank two in [H].
#the nth diagram of the kth row is named dk.n. 
#It returns 0 if the input diagram is not in the list.
#It does not admit mirrored diagrams, added further on.

checkr3b:=function(d)
if 
d[1]=d[3] and d[3]=d[5] and d[1]*d[2]=F(1) and d[1]*d[4]=F(1) and d[1]<>F(1) 
then
return
"d1";
elif 
d[1]=d[5]^2 and d[3]=d[5]^2 and d[1]*d[2]=F(1) and d[3]*d[4]=F(1) and d[5]<>F(1) and d[5]<>-F(1) 
then
return
"d2";
elif 
d[1]=d[3] and d[5]=d[3]^2 and d[1]*d[2]=F(1) and d[5]*d[4]=F(1) and d[1]<>F(1) and d[1]<>-F(1) 
then
return
"d3";
elif 
d[1]=-F(1) and d[3]=d[5] and d[3]*d[2]=F(1) and d[3]*d[4]=F(1) and d[5]<>F(1) and d[5]<>-F(1)
then
return
"d4a";
elif 
d[1]=-F(1) and d[3]=-F(1) and d[4]*d[2]=F(1) and d[5]*d[4]=F(1) and d[5]<>F(1) and d[5]<>-F(1)
then
return
"d4b";
elif 
d[1]=-F(1) and d[3]=d[5]^2 and d[3]*d[2]=F(1) and d[3]*d[4]=F(1) and d[5]<>F(1) and d[5]<>-F(1) and d[5]<>F(4) and d[5]<>-F(4) 
then
return
"d5a";
elif 
d[1]=-F(1) and d[3]=-F(1) and d[4]*d[2]=F(1) and d[2]=d[5]^2 and d[5]<>F(1) and d[5]<>-F(1) and d[5]<>F(4) and d[5]<>-F(4) 
then
return
"d5b";
elif 
d[1]=d[5]^-2 and d[1]*d[2]=F(1) and d[4]*d[2]=F(1) and d[5]<>F(1) and d[5]<>-F(1) and  d[5]<>F(4) and d[5]<>-F(4) 
then
return
"d5c";
elif 
d[1]=-F(1) and d[2]*d[3]=F(1) and d[5]=d[3]^2 and d[4]*d[5]=F(1) and d[3]<>F(1) and d[3]<>-F(1)
then
return
"d6a";
elif 
d[1]=-F(1) and d[3]=-F(1) and d[5]=d[2]^2 and d[4]*d[5]=F(1) and d[2]<>F(1) and d[2]<>-F(1)
then
return
"d6b";
elif 
d[1]=-F(1) and d[3]*d[2]=F(1) and d[5]=d[3]^3 and d[4]*d[5]=F(1) and d[3]<>F(1) and d[3]<>-F(1) and d[3]<>F(3) and d[3]<>F(3)^2
then
return
"d7a";
elif 
d[1]=-F(1) and d[3]=-F(1) and d[5]=d[2]^3 and d[4]*d[5]=F(1) and d[2]<>F(1) and d[2]<>-F(1) and d[2]<>F(3) and d[2]<>F(3)^2
then
return
"d7b";
elif 
d[1]=-d[5]^-3 and d[1]*d[2]=F(1) and d[3]=-F(1) and d[4]=d[5]^-2 and d[5]<>F(1) and d[5]<>-F(1) and d[5]<>-F(3) and d[5]<>-F(3)^2
then
return
"d7c";
elif 
d[1]*d[2]=F(1) and d[5]*d[4]=F(1) and d[5]*d[1]=F(1) and d[3]=-F(1) and d[1]<>F(1) and d[1]<>-F(1)
then
return
"d8a";
elif 
d[4]*d[2]=F(1) and d[1]=-F(1) and d[3]=-F(1) and d[5]=-F(1) and d[2]<>F(1) and d[2]<>-F(1)
then
return
"d8b";
elif 
d[3]*d[2]=F(1) and d[3]*d[4]=F(1) and d[1]=-F(1) and d[5]=-F(1) and d[3]<>F(1) and d[3]<>-F(1)
then
return
"d8c/d";
elif 
d[1]*d[2]=F(1) and d[5]*d[4]=F(1) and d[3]=-F(1) and d[1]*d[5]<>F(1) and d[1]<>F(1) and d[5]<>F(1) 
then
return
"d9";
elif 
d[1]*d[2]=F(1) and d[3]=-F(1) and d[5]=d[1] and d[4]=d[2] and d[1]<>F(1) and d[1]<>-F(1) and d[1]<>F(3) and d[1]<>F(3)^2
then
return
"d10a";
elif 
d[1]*d[2]=F(1) and d[3]=-F(1) and d[5]=d[1]^-2 and d[4]=d[1]^2 and d[1]<>F(1) and d[1]<>-F(1) and d[1]<>F(3) and d[1]<>F(3)^2
then
return
"d10b";
elif 
d[1] in R(3) and d[2]=d[1]^-1 and d[3]=-F1 and d[4]=d[2] and d[5]=d[1]
then
return
"d11";
elif d[1] in R(3) and d[2]=-d[1] and d[3]=-d[1]^-1 and d[4]=-d[1] and d[5]=d[3] then
return 
"d12";
elif d[1] in R(3) and d[5]=-F1 and d[4]=d[1]^-2 and d[2]=d[1]^-1 and d[3]=d[1] then
return 
"d13.1.3";
elif d[1] in R(6) and d[5]=-F1 and d[4]=d[1]^-2 and d[2]=d[1]^-1 and d[3]=d[1] then
return 
"d13.1.6";
elif d[1] in R(3) and d[5]=-F1 and d[4]=d[1]^2 and d[2]=d[1]^-1 and d[3]=-d[1]^-1 then
return 
"d13.2.3";
elif d[1] in R(6) and d[5]=-F1 and d[4]=d[1]^2 and d[2]=d[1]^-1 and d[3]=-d[1]^-1 then
return 
"d13.2.6";
elif d[1]=-F1 and d[5] in R(3) and d[4]=-d[5] and d[2]=d[4] and d[3]=-d[5]^-1 then
return 
"d14.1";
elif d[1]=-F1 and d[5] in R(3) and d[4]=-d[5] and d[2]=d[4]^-1 and d[3]=-F1 then
return 
"d14.2";
elif  
d[5] in R(3) and d[4]=-d[5] and d[2]=d[4]^-1 and d[1]=-d[5] and d[3]=-F1 then
return 
"d14.3";
elif d[1]=-F1 and d[3] in R(3) and d[2]=d[3]^-1 and d[4]=d[3] and d[5]=-F1 then
return 
"d15.1";
elif d[1]=-F1 and d[2] in R(3) and d[3]=d[1] and d[4]=d[2] and d[5]=-F1 then
return 
"d15.2";
elif d[1]=-F1 and d[2] in R(3) and d[3]=-d[2] and d[4]=d[2] and d[5]=-F1 then
return 
"d15.3";
elif d[1]=-F1 and d[3] in R(3) and d[4]=-d[3]^-1 and d[2]=d[3]^-1 and d[5]=-d[3] then
return 
"d16.1";
elif d[1]=-F1 and d[2] in R(3) and d[3]=-F1 and d[4]=-d[2]^-1 and d[5]=-d[2] then
return 
"d16.2";
elif d[1] in R(3) and d[3]=-F1 and d[4]=-d[1]^-1 and d[2]=-F1 and d[5]=-d[1] then
return 
"d16.3";
elif d[1] in R(3) and d[3]=-d[1] and d[4]=-d[1]^-1 and d[2]=-d[1]^-1 and d[5]=-d[1] then
return 
"d16.4";
elif
d[4] in R(3) and d[1]=-F1 and d[2]=d[1] and d[3]=d[1] and d[5]=d[1] then
return 
"d17.1";
elif
d[3] in R(3) and d[1]=-F1 and d[2]=d[1] and d[5]=d[1] and d[4]=d[3]^-1 then
return 
"d17.2";
elif
d[2] in R(3) and d[1]=-F1 and d[3]=d[1] and d[5]=d[1] and d[4]=-d[2] then
return 
"d17.3";
elif
d[4] in R(3) and d[1]=-F1 and d[3]=d[1] and d[5]=d[1] and d[4]=-d[2] then
return 
"d17.3";
elif
d[2] in R(3) and d[3]=d[2] and d[4]=-d[3] and d[1]=-F1 and d[5]=-F1 then
return 
"d17.4";
elif
d[2] in R(3) and d[3]=d[2]^-1 and d[4]=-d[3] and d[1]=-F1 and d[5]=-F1 then
return 
"d17.5";
elif
d[5] in R(3) and d[4]=-d[5] and d[3]=-F1 and d[1]=-F1 and d[2]=-F1 then
return 
"d17.6";
elif
d[2] in R(3) and d[3]=-d[2] and d[4]=-d[2]^-1 and d[1]=-F1 and d[5]=-F1 then
return 
"d17.7";
elif  
d[1] in R(9) and d[2]=d[1]^-1 and d[3]=d[1] and d[4]=d[2] and d[5]=d[1]^-3 then
return 
"d18.1";
elif
d[1] in R(9) and d[2]=d[1]^-1 and d[3]=d[1]^-4 and d[4]=d[1]^4 and d[5]=d[1]^-3 then
return 
"d18.2";
else 
return 0;
fi;
end;


#the function "checkr3p" flips a diagram of rank 3.

checkr3p:=d->checkr3b(Permuted(d,(1,5)(2,4)));


#"checkr3" is analogous to "checkr3b", it admits mirrored diagrams.

checkr3:=function(d)
if checkr3b(d)<>0 then
return
checkr3b(d);
else
return 
checkr3p(d);
fi;
end;

#The function "Hckeckr3" returns 1 if the input diagram (of rank 3, not a triangle) is on the list; returns 0 otherwise. 

Hcheckr3:=function(d)
if checkr3(d)<>0 then
return
1;
else 
return 
0;
fi;
end;

######################################################################################
# 2.1 The list of diagrams of rank 2: the finite, the parametric on q and the parametric functions
######################################################################################

d1:=[q,F(1),r];;
d2:=[q,q^-1,q];;
d3a:=[q,q^-1,-F(1)];;
# q different from -1
d3b:=[-F(1),q^-1,-F(1)];;
d4:=[q,q^-2,q^2];;#B2
d5:=[q,q^-2,-F(1)];;
# q is not a root of order 4
#d5b:=[-q^-1,q^2,-F(1)];;
d6:=[F(3),q^-1,q];;
# q is not a root of order 3
#d6b:=[F(3),F(3)^-1*q,F(3)*q^-1];;
d7:=[F(3),-F(3),-F(1)];;
#d7b:=[F(3)^-1,-F(3)^-1,-F(1)];;
d8a:=[-F(12)^-2,-F(12)^3,-F(12)^2];;
d8b:=[-F(12)^-2,F(12)^-1,-F(1)];;
d8c:=[-F(12)^2,-F(12),-F(1)];;
d8d:=[-F(12)^3,F(12),-F(1)];;
d8e:=[-F(12)^3,-F(12)^-1,-F(1)];;
d9a:=[-F(12)^2,F(12),-F(12)^2];;
d9b:=[-F(12)^2,F(12)^3,-F(1)];;
d9c:=[-F(12)^-1,-F(12)^3,-F(1)];;
d10a:=[-F(9),F(9)^-2,F(9)^3];;
d10b:=[F(9)^3,F(9)^-1,-F(1)];;
d10c:=[-F(9)^2,F(9),-F(1)];;
d11:=[q,q^-3,q^3];;
d12a:=[F(8)^2,F(8),F(8)^-1];;
d12b:=[F(8)^2,-F(8)^-1,-F(1)];;
d12c:=[F(8),-F(8),-F(1)];;
d13a:=[F(24)^6,-F(24)^-1,-F(24)^-4];;
d13b:=[F(24)^6,F(24),F(24)^-1];;
d13c:=[-F(24)^-4,F(24)^5,-F(1)];;
d13d:=[F(24),F(24)^-5,-F(1)];;
d14a:=[F(5),F(5)^2,-F(1)];;
d14b:=[-F(5)^-2,F(5)^-2,-F(1)];;
d15a:=[F(20),F(20)^-3,-F(1)];;
d15b:=[-F(20),-F(20)^-3,-F(1)];;
d15c:=[-F(20)^-2,F(20)^3,-F(1)];;
d15d:=[-F(20)^-2,-F(20)^3,-F(1)];;
d16a:=[-F(15),-F(15)^-3,F(15)^5];;
d16b:=[F(15)^3,-F(15)^4,-F(15)^-4];;
d16c:=[F(15)^5,-F(15)^-2,-F(1)];;
d16d:=[F(15)^3,-F(15)^2,-F(1)];;
d17a:=[-F(7),-F(7)^-3,-F(1)];;
d17b:=[-F(7)^-2,-F(7)^3,-F(1)];;
	
# generic rank 2 diagrams as functions (together with their flips):

d2q:=q->[q,q^-1,q];;
d3aq:=q->[q,q^-1,-F(1)];;
d3apq:=q->[-F(1),q^-1,q];;
# q no es -1
d3bq:=q->[-F(1),q^-1,-F(1)];;
d4q:=q->[q,q^-2,q^2];;
d4pq:=q->[q^2,q^-2,q];;
d5q:=q->[q,q^-2,-F(1)];;
d5pq:=q->[-F(1),q^-2,q];;
# q not of order 4
#d5b:=[-q^-1,q^2,-F(1)];;
d6q:=q->[F(3),q^-1,q];;
d6pq:=q->[q,q^-1,F(3)];;
d11q:=q->[q,q^-3,q^3];;
d11pq:=q->[q^3,q^-3,q];;


#the list of all diagrams:
H:=[
d1,d2,d3a,d3b,d4,d5,d6,d7,d8a,d8b,d8c,d8d,d8e,d9a,d9b,d9c,d10a,d10b,d10c,
d11,d12a,d12b,d12c,d13a,d13b,d13c,d14a,d14b,d15a,d15b,d15c,d15d,d16a,d16b,d16c,d16d,d17a,d17b
];;

#the list of finite (i.e. non-generic) diagrams:
Hf:=[
d7,d8a,d8b,d8c,d8d,d8e,d9a,d9b,d9c,d10a,d10b,d10c,
d12a,d12b,d12c,d13a,d13b,d13c,d14a,d14b,d15a,d15b,d15c,d15d,d16a,d16b,d16c,d16d,d17a,d17b
];;
# the permutations:
fH:=[];;
for d in Hf do
if [d[3],d[2],d[1]]<>d then
Add(fH,[d[3],d[2],d[1]]);
fi;
od;
#both 
Hff:=Concatenation(Hf,fH);;

#list of generic diagrams
Hg:=[d2,d3a,d3b,d4,d5,d11];;
# the permutations:
gH:=[];;
for d in Hg do
if [d[3],d[2],d[1]]<>d then
Add(gH,[d[3],d[2],d[1]]);
fi;
od;
#both:
Hgg:=Concatenation(Hg,gH);;

#list of generic diagrams  as functions
Hgq:=[d2q,d3aq,d3bq,d4q,d5q,d11q];;
#permutations:
Hq:=[d2q,d3aq,d3apq,d3bq,d4q,d4pq,d5q,d5pq,d6q,d6pq,d11q,d11pq];

#parametric diagrams evaluated at roots in Gf
Hqev:=[];
for d in Hq do
for p in Gf do
if d(p)[1]<>F1 and d(p)[2]<>F1 and d(p)[3]<>F1 then
Add(Hqev,d(p));
fi;
od;
od;
Hqev:=Set(Hqev);;

#finite and Gf-evaluated parametric diagrams:
all:=Concatenation(Hff,Hqev);;
all:=Set(all);;


######################################################################################
# 3. Cartan matrices and reflections
######################################################################################

#to compute the entry m(i,j) of the GCM of a diagram d: 
m:=function(d,i,j)
local l,n,dt;
l:=[];
if Number(d)=5 then
dt:=Concatenation(d,[F1]);
else
dt:=d;
fi;
if i=j then
return -2;
elif j=i+1 or i=j+1 then
for n in [0..10] do
if 
dt[2*i-1]^n*dt[i+j-1]=F(1)
then 
Add(l,n);
fi;
od;
if dt[2*i-1] in Rp then
Add(l,Order(dt[2*i-1])-1);
fi;
if l<>[] then return
Minimum(l);
else return 0;
fi;
elif j=i+2 or i=j+2 then
for n in [0..10] do
if 
dt[2*i-1]^n*dt[i+j+2]=F(1)
then 
Add(l,n);
fi;
od;
if dt[2*i-1] in Rp then
Add(l,Order(dt[2*i-1])-1);
fi;
if l<>[] then return
Minimum(l);
else return 0;
fi;
else return
0;
fi;
end;

#the function "mm" gives the list of values of m(i,j) for each i,j.
mm:=function(dt)
local d;
if Number(dt)=5 then
d:=Concatenation(dt,[F1]);
else
d:=dt;
fi;
return
[m(d,1,2),m(d,2,1),m(d,2,3),m(d,3,2),m(d,1,3),m(d,3,1)];
end;


#reflections

ss:=function(i,d)
local dt;
if Number(d)=5 then
dt:=Concatenation(d,[F1]);
else
dt:=d;
fi;
if i=1 then 
return
[dt[1],dt[1]^(2*m(dt,1,1)*m(dt,1,2))*dt[1]^(2*m(dt,1,2))*dt[2]^m(dt,1,1)*dt[2],(dt[1]^m(dt,1,2)*dt[2])^m(dt,1,2)*dt[3],dt[1]^(2*m(dt,1,2)*m(dt,1,3))*dt[2]^m(dt,1,3)*dt[6]^m(dt,1,2)*dt[4],(dt[1]^m(dt,1,3)*dt[6])^m(dt,1,3)*dt[5],dt[1]^(2*m(dt,1,1)*m(dt,1,3))*dt[1]^(2*m(dt,1,3))*dt[6]^m(dt,1,1)*dt[6]];
elif i=2 then 
return
[(dt[3]^m(dt,2,1)*dt[2])^m(dt,2,1)*dt[1],dt[3]^(2*m(dt,2,1)*m(dt,2,2))*dt[2]^m(dt,2,2)*dt[3]^(2*m(dt,2,1))*dt[2],dt[3],dt[3]^(2*m(dt,2,2)*m(dt,2,3))*dt[3]^(2*m(dt,2,3))*dt[4]^m(dt,2,2)*dt[4],(dt[3]^m(dt,2,3)*dt[4])^m(dt,2,3)*dt[5],dt[3]^(2*m(dt,2,1)*m(dt,2,3))*dt[2]^m(dt,2,3)*dt[4]^m(dt,2,1)*dt[6]];
elif i=3 then 
return
[(dt[5]^m(dt,3,1)*dt[6])^m(dt,3,1)*dt[1],dt[5]^(2*m(dt,3,1)*m(dt,3,2))*dt[6]^m(dt,3,2)*dt[4]^m(dt,3,1)*dt[2],(dt[5]^m(dt,3,2)*dt[4])^m(dt,3,2)*dt[3],dt[5]^(2*m(dt,3,2)*m(dt,3,3))*dt[4]^m(dt,3,3)*dt[5]^(2*m(dt,3,2))*dt[4],dt[5],dt[5]^(2*m(dt,3,1)*m(dt,3,3))*dt[6]^m(dt,3,3)*dt[5]^(2*m(dt,3,1))*dt[6]];
fi;
end;

s:=function(i,d)
if ss(i,d)[2]<>F1 and ss(i,d)[4]<>F1 then
return
ss(i,d);
elif ss(i,d)[2]=F1 then
return
Permuted(ss(i,d),(1,5,3)(2,6,4));
elif ss(i,d)[4]=F1 then
return
Permuted(ss(i,d),(1,3,5)(2,4,6));
fi;
end;



#########
# 4. Diagrams associated to hyperplanes
#########

# diagram associated to i*alpha1+j*alpha2+k*alpha3 and p*alpha1+q*alpha2+r*alpha3
dab:=function(dt,i,j,k,p,q,r)
local d;
if Number(dt)=5 then
d:=Concatenation(dt,[F1]);
else
d:=dt;
fi;
return
[d[1]^(i^2)*d[3]^(j^2)*d[5]^(k^2)*d[2]^(i*j)*d[6]^(i*k)*d[4]^(j*k),
d[1]^(2*i*p)*d[3]^(2*j*q)*d[5]^(2*k*r)*d[2]^(i*q+j*p)*d[6]^(i*r+k*p)*d[4]^(j*r+k*q),
d[1]^(p^2)*d[3]^(q^2)*d[5]^(r^2)*d[2]^(p*q)*d[6]^(p*r)*d[4]^(q*r)];
end;


##########
# 4.1 Diagrams for Criterium 1
##########

# diagram associated to  alpha1 and  k*alpha2+alpha3
w:=function(d,k)
return
dab(d,1,0,0,0,k,1);
end;

# diagram associated to alpha1 and  alpha2+k*alpha3
w1:=function(d,v)
return
dab(d,1,0,0,0,1,v);
end;

# diagram associated to k*alpha1+alpha2 and  alpha3
w2:=function(d,h)
return
dab(d,h,1,0,0,0,1);
end;

# diagram associated to  alpha1+k*alpha2 and  alpha3
w3:=function(d,p)
return
dab(d,1,p,0,0,0,1);
end;


# diagram associated to alpha2 and  k*alpha1+alpha3
w4:=function(d,r)
return
dab(d,0,1,0,r,0,1);
end;

# diagram associated to alpha2 and  alpha1+k*alpha3
w5:=function(d,s)
return
dab(d,0,1,0,1,0,s);
end;


##########
# 4.2 Diagrams for Criterium 2
##########


# diagram associated to alpha1+alpha2+alpha3 and  k*alpha1+alpha2 
w412:=function(d,k)
return
dab(d,1,1,1,k,1,0);
end;

# diagram associates to alpha1+alpha2+alpha3 and  k*alpha2+alpha1 
w421:=function(d,k)
return
dab(d,1,1,1,1,k,0);
end;

# diagram associated to alpha1+alpha2+alpha3 and  k*alpha2+alpha3 
w423:=function(d,k)
return
dab(d,1,1,1,0,k,1);
end;

# diagram associated to alpha1+alpha2+alpha3 and  k*alpha3+alpha2 
w432:=function(d,k)
return
dab(d,1,1,1,0,1,k);
end;

# diagram associated to alpha1+alpha2+alpha3 and  k*alpha1+alpha3 
w413:=function(d,k)
return
dab(d,1,1,1,k,0,1);
end;

# diagram associated to alpha1+alpha2+alpha3 and  k*alpha3+alpha1 
w431:=function(d,k)
return
dab(d,1,1,1,1,0,k);
end;


##########
# 4.3 Diagrams for Criterium 3
##########

# diagram associated to alpha1+alpha2 and alpha2+alpha3
w1223:=function(d)
return
dab(d,1,1,0,0,1,1);
end;

# diagram associated to alpha1+alpha2 and alpha1+alpha3
w1213:=function(d)
return
dab(d,1,1,0,1,0,1);
end;

# diagram associated to alpha1+alpha3 and alpha2+alpha3
w1323:=function(d)
return
dab(d,1,0,1,0,1,1);
end;


###################
# 5. Tests and Criteria
###################


####################
# 5.1 Cartan
####################

#to check if a diagram is Cartan.
cartan:=function(dt)
local d;
if Number(dt)=5 then
d:=Concatenation(dt,[F1]);
else
d:=dt;
fi;
if 
d[1]^m(d,1,2)*d[2]=F1 and d[1]^m(d,1,3)*d[6]=F1 and d[3]^m(d,2,1)*d[2]=F1 and d[3]^m(d,2,3)*d[4]=F1 and d[5]^m(d,3,2)*d[4]=F1 and d[5]^m(d,3,1)*d[6]=F1 
then return 
1;
else return
0;
fi;
end;


#####
# 5.2 Criterium 1
#####

#the function "testk", depending on a diagram d and a natural number k applies the function "Hcheck" (which returns either 1 or 0 dependending a given rank 2 diagram is on the list or not) for the diagram associated to the roots alpha1 and k*alpha2+alpha3

testk:=function(d,k)
return
Hcheck(w(d,k));
end;

# idem as above, for alpha1 y alpha2+k*alpha3
testv:=function(d,v)
return
Hcheck(w1(d,v));
end;


# idem as above, for  a k*alpha1+alpha2 y alpha3
testh:=function(d,h)
return
Hcheck(w2(d,h));
end;

# idem as above, for  alpha1+k*alpha2 y alpha3
testp:=function(d,p)
return
Hcheck(w3(d,p));
end;

# idem as above, for  k*alpha1+alpha3 y alpha2
testr:=function(d,r)
return
Hcheck(w4(d,r));
end;

# idem as above, for  alpha1+k*alpha3 y alpha2
tests:=function(d,s)
return
Hcheck(w5(d,s));
end;


##################
# 5.2.1 The test for Criterium 1
##################

test1:=function(d)
local y,t,k,v,n,j,h,p,r,s;
if Number(d)=5 then
y:=Concatenation(d,[F1]);
else
y:=d;
fi;
k:=1;
v:=1;
h:=1;
p:=1;
r:=1;
s:=1;
	for j in [1..m(y,2,3)] do
		if testk(y,j)=1 then
		k:=k+1;
		fi;
	od;
		if k=m(y,2,3)+1 then
			for j in [1..m(y,3,2)] do
				if testv(y,j)=1 then
				v:=v+1;
				fi;
			od;
		if v=m(y,3,2)+1 then
			for j in [1..m(y,1,2)] do
			if testh(y,j)=1 then
			h:=h+1;
			fi;
			od;
			if h=m(y,1,2)+1 then
				for j in [1..m(y,2,1)] do
				if testp(y,j)=1 then
				p:=p+1;
				fi;
				od;
				if p=m(y,2,1)+1 then
		for j in [1..m(y,1,3)] do
			if testr(y,j)=1 then
			r:=r+1;
			fi;
			od;
			if r=m(y,1,3)+1 then
				for j in [1..m(y,3,1)] do
				if tests(y,j)=1 then
				s:=s+1;
				fi;
				od;
				if s=m(y,3,1)+1 then		
		return 
		1;
		else return
		0;
		fi;
	else return
	0;
	fi;
else return
	0;
	fi;
else return
	0;
	fi;
		else return
		0;
		fi;
else return
0;
fi;
end;

#####
# 5.3 Criterium 2
#####


test123_12d:=function(d)
local k,j,n;
n:=m(d,1,2);
if n<2 then return
1;
else
k:=2;
for j in [2..n] do
if 
Hcheck(w412(d,k))=1
then
k:=k+1;
fi;
od;
if k=n+1 then
return
1;
else return
0;
fi;
fi;
end;

test123_21d:=function(d)
local k,j,n;
n:=m(d,2,1);
if n<2 then return
1;
else
k:=2;
for j in [2..n] do
if 
Hcheck(w421(d,k))=1
then
k:=k+1;
fi;
od;
if k=n+1 then
return
1;
else return
0;
fi;
fi;
end;

test123_23d:=function(d)
local k,j,n;
n:=m(d,2,3);
if n<2 then return
1;
else
k:=2;
for j in [2..n] do
if 
Hcheck(w423(d,k))=1
then
k:=k+1;
fi;
od;
if k=n+1 then
return
1;
else return
0;
fi;
fi;
end;

test123_32d:=function(d)
local k,j,n;
n:=m(d,3,2);
if n<2 then return
1;
else
k:=2;
for j in [2..n] do
if 
Hcheck(w432(d,k))=1
then
k:=k+1;
fi;
od;
if k=n+1 then
return
1;
else return
0;
fi;
fi;
end;


test123_13d:=function(d)
local k,j,n;
n:=m(d,1,3);
if n<2 then return
1;
else
k:=2;
for j in [2..n] do
if 
Hcheck(w413(d,k))=1
then
k:=k+1;
fi;
od;
if k=n+1 then
return
1;
else return
0;
fi;
fi;
end;

test123_31d:=function(d)
local k,j,n;
n:=m(d,3,1);
if n<2 then return
1;
else
k:=2;
for j in [2..n] do
if 
Hcheck(w431(d,k))=1
then
k:=k+1;
fi;
od;
if k=n+1 then
return
1;
else return
0;
fi;
fi;
end;

##################
# 5.3.1 The test for Criterium 2
##################

test2:=function(d)
local dt;
if Number(d)=5 then
dt:=Concatenation(d,[F1]);
else
dt:=d;
fi;
if test123_12d(dt)=0 then 
return
0;
elif test123_21d(dt)=0 then 
return
0;
elif test123_23d(dt)=0 then 
return
0;
elif test123_32d(dt)=0 then 
return
0;
elif test123_13d(dt)=0 then 
return
0;
elif test123_31d(dt)=0 then 
return
0;
else return
1;
fi;
end;


##################
# 5.4 Criterium 3
##################


test1223:=function(d)
if m(d,1,2)*m(d,2,3)<>0 then
return 
Hcheck(w1223(d));
else return
1;
fi;
end;

test1213:=function(d)
if m(d,1,2)*m(d,1,3)<>0 then
return 
Hcheck(w1213(d));
else return
1;
fi;
end;

test1323:=function(d)
if m(d,1,3)*m(d,2,3)<>0 then
return 
Hcheck(w1323(d));
else return
1;
fi;
end;


##################
# 5.4.1 The test for Criterium 3
##################


test3:=function(d)
if test1223(d)=0 then 
return
0;
elif test1213(d)=0 then 
return
0;
elif test1323(d)=0 then 
return
0;
else return
1;
fi;
end;

#########################
# 5.5 All tests together
#########################

#Tests for Criteria 1 to 3:

test123:=function(d)
if test1(d)=0 then
return 0;
elif test2(d)=0 then 
return 0;
elif test3(d)=0 then 
return 0;
else return
1;
fi;
end;


###########################
# 5.5.1 The complete test: Cartan, 1 to 3, and reflections.
###########################

testall:=function(d)
#test 1 to 3:
if test123(d)=0 then 
return 0;
#cartan:
elif cartan(d)=1 then
return 0;
#test 1 to 3 for reflections:
elif test123(s(1,d))=0 then 
return 0;
elif test123(s(2,d))=0 then 
return 0;
elif test123(s(3,d))=0 then 
return 0;
elif test123(s(1,s(2,d)))=0 then 
return 0;
elif test123(s(1,s(3,d)))=0 then 
return 0;
elif test123(s(2,s(1,d)))=0 then 
return 0;
elif test123(s(2,s(3,d)))=0 then 
return 0;
elif test123(s(3,s(2,d)))=0 then 
return 0;
elif test123(s(3,s(1,d)))=0 then 
return 0;
elif test123(s(2,s(1,s(2,d))))=0 then 
return 0;
elif test123(s(3,s(1,s(2,d))))=0 then 
return 0;
elif test123(s(2,s(1,s(3,d))))=0 then 
return 0;
elif test123(s(3,s(1,s(3,d))))=0 then 
return 0;
elif test123(s(1,s(2,s(1,d))))=0 then 
return 0;
elif test123(s(3,s(2,s(1,d))))=0 then 
return 0;
elif test123(s(1,s(2,s(3,d))))=0 then 
return 0;
elif test123(s(3,s(2,s(3,d))))=0 then 
return 0;
elif test123(s(1,s(3,s(2,d))))=0 then 
return 0;
elif test123(s(2,s(3,s(2,d))))=0 then 
return 0;
elif test123(s(1,s(3,s(1,d))))=0 then 
return 0;
elif test123(s(2,s(3,s(1,d))))=0 then 
return 0;
else return
1;
fi;
end;

###############################################
# 6. To remove permutations and/or reflections from a list.
###############################################

per:=function(l)
local d,ll,lt,lf;
lt:=[];
lf:=[];
for d in l do
if Number(d)=5 then
Add(lt,Concatenation(d,[F1]));
else
Add(lt,d);
fi;
od;
ll:=[lt[1]];
for d in lt do
if d in ll 
or Permuted(d,(1,3,5)(2,4,6)) in ll or Permuted(d,(1,5,3)(2,6,4)) in ll 
or Permuted(d,(3,5)(2,6)) in ll or Permuted(d,(1,5)(2,4)) in ll or Permuted(d,(1,3)(4,6)) in ll
or s(1,d) in ll  
or Permuted(s(1,d),(1,3,5)(2,4,6)) in ll or Permuted(s(1,d),(1,5,3)(2,6,4)) in ll 
or Permuted(s(1,d),(3,5)(2,6)) in ll or Permuted(s(1,d),(1,5)(2,4)) in ll or Permuted(s(1,d),(1,3)(4,6)) in ll
or s(2,d) in ll 
or Permuted(s(2,d),(1,3,5)(2,4,6)) in ll or Permuted(s(2,d),(1,5,3)(2,6,4)) in ll 
or Permuted(s(2,d),(3,5)(2,6)) in ll or Permuted(s(2,d),(1,5)(2,4)) in ll or Permuted(s(2,d),(1,3)(4,6)) in ll
or s(3,d) in ll
or Permuted(s(3,d),(1,3,5)(2,4,6)) in ll or Permuted(s(3,d),(1,5,3)(2,6,4)) in ll 
or Permuted(s(3,d),(3,5)(2,6)) in ll or Permuted(s(3,d),(1,5)(2,4)) in ll or Permuted(s(3,d),(1,3)(4,6)) in ll
then
else
Add(ll,d);
fi;
od;
for d in ll do
if d[6]=F1 then
Remove(d,6);
Add(lf,d);
else
Add(lf,d);
fi;
od;
return
Set(lf);
end;


